/*
   ipc.c -- PIKT ipc (interprocess communication) routines

   $Header: /home/cvs/ucgsb/pikt/src/lib/ipc.c,v 1.3 1999/10/04 05:14:45 harlan Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA

   ----------------------------------------------------------------------------

   We might redo the striplex()-to-*parse() coordination using signals, but
   IPC and pipes are more portable.

 */

#include "common.h"

int p2cfd[MAXIFS+1][2], c2pfd[MAXIFS+1][2];

void ipcopen(int);
void ipcclose(int);
void tell_parent(int);
void tell_child(int);
void wait_parent(int);
void wait_child(int);

/* open the parent-to-child and child-to-parent channels
 */
void
ipcopen (int incnum)
{
	if (pipe(p2cfd[incnum]) < 0 || pipe(c2pfd[incnum]) < 0)
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in ipcopen(), pipe() error");
	return;
}

/* close the parent-to-child and child-to-parent channels
 */
void
ipcclose(int incnum)
{
	close(p2cfd[incnum][0]);
	close(p2cfd[incnum][1]);
	close(c2pfd[incnum][0]);
	close(c2pfd[incnum][1]);
	return;
}

/* the child process signals the parent
 */
void
tell_parent (int incnum)
{
	if (write(c2pfd[incnum][1], "c", 1) != 1)
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in tell_parent(), write() error");
	return;
}

/* the child process waits for the parent's signal
 */
void
wait_parent (int incnum)
{
	static char c;

	if (read(p2cfd[incnum][0], &c, 1) != 1)
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in wait_parent(), read() error");
	if (c != 'p')
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in wait_parent(), invalid signal");
	return;
}

/* the parent process signals the child
 */
void
tell_child (int incnum)
{
	if (write(p2cfd[incnum][1], "p", 1) != 1)
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in tell_child(), write() error");
	return;
}

/* the parent process waits for the child's signal; we allow for the
   possibility that the child has already died, perhaps closing down the
   channel
 */
void
wait_child (int incnum)
{
	static char c;
	int n;

	if ((n = read(c2pfd[incnum][0], &c, 1)) < 0)
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in wait_child(), read() error");
	else if (n == 0)
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in wait_child(), EOF read()");
	else if (/* (n == 1) && */ (c != 'c'))
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in wait_child(), invalid signal");
	return;
}
