%{

/*
   cmd.l -- PIKT piktc command-line lexer

   $Header: /home/cvs/ucgsb/pikt/src/piktc/cmd.l,v 1.23 1999/10/31 05:20:21 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include "../lib/common.h"
#include "piktc.h"
#include "cmd.tab.h"

char _cmdlex_[] = "cmdlex";

char errmsg[MAXLINE+1];
signed int cmdinput(void);

char ALL[] = "+A all +P all +F all +O all +H all ";
int ii;

#define RETURN(token) cmdlval.sval = mystrdup(yytext, _cmdlex_); return token

#undef YY_INPUT
#define YY_INPUT(buf, result, max_size)	{ \
		signed char c = cmdinput(); \
		result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
	}

#define CHKCONFLICT(A, B, FL)	if (FL) {\
					fprintf(stderr, "conflicting options: %c %c\n", A, B);\
					fflush(stderr);\
					bailout = TRUE;\
				}

void wrapstr(char **);
void unwrapstr(char **);
int hasmacro(char *);

%}

W		[ \t]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]

%x mS pS DS CS AS PS FS OS HS

%option noyywrap

%%

<INITIAL,DS,AS,PS,FS,OS,HS>"-"	{ BEGIN(mS);
				  return _MINUS;
				}
<INITIAL,DS,AS,PS,FS,OS,HS>"+"	{ BEGIN(pS);
				  return _PLUS;
				}

<mS,pS>(A|S)f?			{ BEGIN(AS);
				  if (!postscan) {
					CHKCONFLICT((yytext[0] == 'A' ? 'A' : 'S'), 'c', ischeck);
					CHKCONFLICT((yytext[0] == 'A' ? 'A' : 'S'), 'C', isdoublecheck);
					CHKCONFLICT((yytext[0] == 'A' ? 'A' : 'S'), 'h', ishelp);
					if (yytext[1] == 'f')
						stripalm = FALSE;
					rcfbs |= (1L << ALMCFG);
					rcfbs |= (1L << ALTCFG);
				  }
				  return _ALTOPT;  /* _SCTOPT */
				}

<mS>c|C				{ if (yytext[0] == 'c') {
					if (!postscan) {
						CHKCONFLICT('c', 'C', isdoublecheck);
						ischeck = TRUE;
					}
				  } else {
					if (!postscan) {
						CHKCONFLICT('C', 'c', ischeck);
						isdoublecheck = TRUE;
					}
				  }
				  if (!postscan) {
					CHKCONFLICT('c', 'd', isdisable);
					CHKCONFLICT('c', 'e', isenable);
					CHKCONFLICT('c', 'f', isdiff);
					CHKCONFLICT('c', 'h', ishelp);
					CHKCONFLICT('c', 'i', isinstall);
					CHKCONFLICT('c', 'k', iskill_piktd);
					CHKCONFLICT('c', 'K', iskill_piktc_svc);
					CHKCONFLICT('c', 'l', islist);
					CHKCONFLICT('c', 'r', isrestart_piktd);
					CHKCONFLICT('c', 'R', isrestart_piktc_svc);
					CHKCONFLICT('c', 's', isshow);
					CHKCONFLICT('c', 't', isdelete);
					CHKCONFLICT('c', 'x', isexecwait);
					CHKCONFLICT('c', 'X', isexec);
				  }
				  return _OPT;
				}

<pS>C				{ BEGIN(CS);
				  if (!postscan) {
					CHKCONFLICT('C', 'c', ischeck);
					CHKCONFLICT('C', 'C', isdoublecheck);
					CHKCONFLICT('C', 'd', isdisable);
					CHKCONFLICT('C', 'e', isenable);
					CHKCONFLICT('C', 'f', isdiff);
					CHKCONFLICT('C', 'h', ishelp);
					CHKCONFLICT('C', 'i', isinstall);
					CHKCONFLICT('C', 'k', iskill_piktd);
					CHKCONFLICT('C', 'K', iskill_piktc_svc);
					CHKCONFLICT('C', 'l', islist);
					CHKCONFLICT('C', 'r', isrestart_piktd);
					CHKCONFLICT('C', 'R', isrestart_piktc_svc);
					CHKCONFLICT('C', 's', isshow);
					CHKCONFLICT('C', 't', isdelete);
				  }
				  return _CMDOPT;
				}

<mS>d				{ if (!postscan) {
					isdisable = TRUE;
					CHKCONFLICT('d', 'c', ischeck);
					CHKCONFLICT('d', 'C', isdoublecheck);
					CHKCONFLICT('d', 'e', isenable);
					CHKCONFLICT('d', 'f', isdiff);
					CHKCONFLICT('d', 'h', ishelp);
					CHKCONFLICT('d', 'i', isinstall);
					CHKCONFLICT('d', 'K', iskill_piktc_svc);
					CHKCONFLICT('d', 'l', islist);
					CHKCONFLICT('d', 'R', isrestart_piktc_svc);
					CHKCONFLICT('d', 's', isshow);
					CHKCONFLICT('d', 'x', isexecwait);
					CHKCONFLICT('d', 'X', isexec);
				  }
				  return _OPT;
				}

<mS,pS>D			{ BEGIN(DS);
				  if (!postscan) {
					CHKCONFLICT('D', 'h', ishelp);
					rcfbs |= (1L << DEFCFG);
				  }
				  return _DEFOPT;
				}

<mS>e				{ if (!postscan) {
					isenable = TRUE;
					CHKCONFLICT('e', 'c', ischeck);
					CHKCONFLICT('e', 'C', isdoublecheck);
					CHKCONFLICT('e', 'd', isdisable);
					CHKCONFLICT('e', 'f', isdiff);
					CHKCONFLICT('e', 'h', ishelp);
					CHKCONFLICT('e', 'K', iskill_piktc_svc);
					CHKCONFLICT('e', 'l', islist);
					CHKCONFLICT('e', 'R', isrestart_piktc_svc);
					CHKCONFLICT('e', 's', isshow);
					CHKCONFLICT('e', 't', isdelete);
					CHKCONFLICT('e', 'x', isexecwait);
					CHKCONFLICT('e', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>f				{ if (!postscan) {
					isdiff = TRUE;
					CHKCONFLICT('f', 'c', ischeck);
					CHKCONFLICT('f', 'C', isdoublecheck);
					CHKCONFLICT('f', 'd', isdisable);
					CHKCONFLICT('f', 'e', isenable);
					CHKCONFLICT('f', 'h', ishelp);
					CHKCONFLICT('f', 'i', isinstall);
					CHKCONFLICT('f', 'k', iskill_piktd);
					CHKCONFLICT('f', 'K', iskill_piktc_svc);
					CHKCONFLICT('f', 'l', islist);
					CHKCONFLICT('f', 'r', isrestart_piktd);
					CHKCONFLICT('f', 'R', isrestart_piktc_svc);
					CHKCONFLICT('f', 's', isshow);
					CHKCONFLICT('f', 't', isdelete);
					CHKCONFLICT('f', 'x', isexecwait);
					CHKCONFLICT('f', 'X', isexec);
				  }
				  return _OPT;
				}

<mS,pS>(F)f?			{ BEGIN(FS);
				  if (!postscan) {
					CHKCONFLICT('F', 'c', ischeck);
					CHKCONFLICT('F', 'C', isdoublecheck);
					CHKCONFLICT('F', 'h', ishelp);
					if (yytext[1] == 'f')
						stripfil = FALSE;
					rcfbs |= (1L << FILCFG);
				  }
				  return _FILOPT;
				}

<mS>G				{ if (!postscan)
					isdebug = TRUE;
				  return _OPT;
				}

<mS>h				{ if (!postscan) {
					ishelp = TRUE;
					CHKCONFLICT('h', 'c', ischeck);
					CHKCONFLICT('h', 'C', isdoublecheck);
					CHKCONFLICT('h', 'd', isdisable);
					CHKCONFLICT('h', 'e', isenable);
					CHKCONFLICT('h', 'f', isdiff);
					CHKCONFLICT('h', 'i', isinstall);
					CHKCONFLICT('h', 'k', iskill_piktd);
					CHKCONFLICT('h', 'K', iskill_piktc_svc);
					CHKCONFLICT('h', 'l', islist);
					CHKCONFLICT('h', 'r', isrestart_piktd);
					CHKCONFLICT('h', 'R', isrestart_piktc_svc);
					CHKCONFLICT('h', 's', isshow);
					CHKCONFLICT('h', 't', isdelete);
					CHKCONFLICT('h', 'v', isverbose);
					CHKCONFLICT('h', 'x', isexecwait);
					CHKCONFLICT('h', 'X', isexec);
				  }
				  return _OPT;
				}

<mS,pS>H			{ BEGIN(HS);
				  if (!postscan)
					CHKCONFLICT('H', 'h', ishelp);
				  return _HSTOPT;
				}

<mS>i				{ if (!postscan) {
					isinstall = TRUE;
					CHKCONFLICT('i', 'c', ischeck);
					CHKCONFLICT('i', 'C', isdoublecheck);
					CHKCONFLICT('i', 'd', isdisable);
					CHKCONFLICT('i', 'f', isdiff);
					CHKCONFLICT('i', 'h', ishelp);
					CHKCONFLICT('i', 'k', iskill_piktd);
					CHKCONFLICT('i', 'K', iskill_piktc_svc);
					CHKCONFLICT('i', 'l', islist);
					CHKCONFLICT('i', 'R', isrestart_piktc_svc);
					CHKCONFLICT('i', 's', isshow);
					CHKCONFLICT('i', 't', isdelete);
					CHKCONFLICT('i', 'x', isexecwait);
					CHKCONFLICT('i', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>k				{ if (!postscan) {
					iskill_piktd = TRUE;
					CHKCONFLICT('k', 'c', ischeck);
					CHKCONFLICT('k', 'C', isdoublecheck);
					CHKCONFLICT('k', 'f', isdiff);
					CHKCONFLICT('k', 'h', ishelp);
					CHKCONFLICT('k', 'i', isinstall);
					CHKCONFLICT('k', 'l', islist);
					CHKCONFLICT('k', 'r', isrestart_piktd);
					CHKCONFLICT('k', 's', isshow);
					CHKCONFLICT('k', 'x', isexecwait);
					CHKCONFLICT('k', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>K				{ if (!postscan) {
					iskill_piktc_svc = TRUE;
					CHKCONFLICT('K', 'c', ischeck);
					CHKCONFLICT('K', 'C', isdoublecheck);
					CHKCONFLICT('K', 'd', isdisable);
					CHKCONFLICT('K', 'e', isenable);
					CHKCONFLICT('K', 'f', isdiff);
					CHKCONFLICT('K', 'h', ishelp);
					CHKCONFLICT('K', 'i', isinstall);
					CHKCONFLICT('K', 'l', islist);
					CHKCONFLICT('K', 'r', isrestart_piktd);
					CHKCONFLICT('K', 'R', isrestart_piktc_svc);
					CHKCONFLICT('K', 's', isshow);
					CHKCONFLICT('K', 't', isdelete);
					CHKCONFLICT('K', 'x', isexecwait);
					CHKCONFLICT('K', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>l				{ if (!postscan) {
					islist = TRUE;
					CHKCONFLICT('l', 'c', ischeck);
					CHKCONFLICT('l', 'C', isdoublecheck);
					CHKCONFLICT('l', 'd', isdisable);
					CHKCONFLICT('l', 'e', isenable);
					CHKCONFLICT('l', 'f', isdiff);
					CHKCONFLICT('l', 'h', ishelp);
					CHKCONFLICT('l', 'i', isinstall);
					CHKCONFLICT('l', 'k', iskill_piktd);
					CHKCONFLICT('l', 'K', iskill_piktc_svc);
					CHKCONFLICT('l', 'R', isrestart_piktc_svc);
					CHKCONFLICT('l', 's', isshow);
					CHKCONFLICT('l', 'x', isexecwait);
					CHKCONFLICT('l', 'X', isexec);
				  }
				  return _OPT;
				}

<mS,pS>(O)f?			{ BEGIN(OS);
				  if (!postscan) {
					CHKCONFLICT('O', 'c', ischeck);
					CHKCONFLICT('O', 'C', isdoublecheck);
					CHKCONFLICT('O', 'h', ishelp);
					if (yytext[1] == 'f')
						stripobj = FALSE;
					rcfbs |= (1L << OBJCFG);
				  }
				  return _OBJOPT;
				}

<mS,pS>(P)f?			{ BEGIN(PS);
				  if (!postscan) {
					CHKCONFLICT('P', 'c', ischeck);
					CHKCONFLICT('P', 'C', isdoublecheck);
					CHKCONFLICT('P', 'h', ishelp);
					if (yytext[1] == 'f')
						stripprg = FALSE;
					rcfbs |= (1L << PRGCFG);
				  }
				  return _PRGOPT;
				}

<mS>r				{ if (!postscan) {
					isrestart_piktd = TRUE;
					CHKCONFLICT('r', 'c', ischeck);
					CHKCONFLICT('r', 'C', isdoublecheck);
					CHKCONFLICT('r', 'f', isdiff);
					CHKCONFLICT('r', 'h', ishelp);
					CHKCONFLICT('r', 'k', iskill_piktd);
					CHKCONFLICT('r', 'K', iskill_piktc_svc);
					CHKCONFLICT('r', 'l', islist);
					CHKCONFLICT('r', 'x', isexecwait);
					CHKCONFLICT('r', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>R				{ if (!postscan) {
					isrestart_piktc_svc = TRUE;
					CHKCONFLICT('R', 'c', ischeck);
					CHKCONFLICT('R', 'C', isdoublecheck);
					CHKCONFLICT('R', 'd', isdisable);
					CHKCONFLICT('R', 'e', isenable);
					CHKCONFLICT('R', 'f', isdiff);
					CHKCONFLICT('R', 'h', ishelp);
					CHKCONFLICT('R', 'i', isinstall);
					CHKCONFLICT('R', 'K', iskill_piktc_svc);
					CHKCONFLICT('R', 'l', islist);
					CHKCONFLICT('R', 's', isshow);
					CHKCONFLICT('R', 't', isdelete);
					CHKCONFLICT('R', 'x', isexecwait);
					CHKCONFLICT('R', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>s				{ if (!postscan) {
					isshow = TRUE;
					CHKCONFLICT('s', 'c', ischeck);
					CHKCONFLICT('s', 'C', isdoublecheck);
					CHKCONFLICT('s', 'd', isdisable);
					CHKCONFLICT('s', 'e', isenable);
					CHKCONFLICT('s', 'f', isdiff);
					CHKCONFLICT('s', 'h', ishelp);
					CHKCONFLICT('s', 'i', isinstall);
					CHKCONFLICT('s', 'k', iskill_piktd);
					CHKCONFLICT('s', 'K', iskill_piktc_svc);
					CHKCONFLICT('s', 'l', islist);
					CHKCONFLICT('s', 'R', isrestart_piktc_svc);
					CHKCONFLICT('s', 't', isdelete);
					CHKCONFLICT('s', 'x', isexecwait);
					CHKCONFLICT('s', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>t				{ if (!postscan) {
					isdelete = TRUE;
					CHKCONFLICT('t', 'c', ischeck);
					CHKCONFLICT('t', 'C', isdoublecheck);
					CHKCONFLICT('t', 'e', isenable);
					CHKCONFLICT('t', 'f', isdiff);
					CHKCONFLICT('t', 'h', ishelp);
					CHKCONFLICT('t', 'i', isinstall);
					CHKCONFLICT('t', 'K', iskill_piktc_svc);
					CHKCONFLICT('t', 'R', isrestart_piktc_svc);
					CHKCONFLICT('t', 's', isshow);
					CHKCONFLICT('t', 'x', isexecwait);
					CHKCONFLICT('t', 'X', isexec);
				  }
				  return _OPT;
				}

<mS>T				{ if (!postscan) {
					istest = TRUE;
					/* isdebug = TRUE; */
				  }
				  return _OPT;
				}

<mS>v				{ if (!postscan) {
					isverbose = TRUE;
					CHKCONFLICT('v', 'h', ishelp);
				  }
				  return _OPT;
				}

<mS>x|X				{ if (yytext[0] == 'x') {
					if (!postscan) {
						CHKCONFLICT('x', 'X', isexec);
						isexecwait = TRUE;
					}
				  } else {
					if (!postscan) {
						CHKCONFLICT('X', 'x', isexecwait);
						isexec = TRUE;
					}
				  }
				  if (!postscan) {
					CHKCONFLICT('x', 'c', ischeck);
					CHKCONFLICT('x', 'C', isdoublecheck);
					CHKCONFLICT('x', 'd', isdisable);
					CHKCONFLICT('x', 'e', isenable);
					CHKCONFLICT('x', 'f', isdiff);
					CHKCONFLICT('x', 'h', ishelp);
					CHKCONFLICT('x', 'i', isinstall);
					CHKCONFLICT('x', 'k', iskill_piktd);
					CHKCONFLICT('x', 'K', iskill_piktc_svc);
					CHKCONFLICT('x', 'l', islist);
					CHKCONFLICT('x', 'r', isrestart_piktd);
					CHKCONFLICT('x', 'R', isrestart_piktc_svc);
					CHKCONFLICT('x', 's', isshow);
					CHKCONFLICT('x', 't', isdelete);
				  }
				  return _OPT;
				}

<mS,pS>{W}			{ BEGIN(INITIAL); }

<mS,pS>.			{ if (!postscan) {	/* should never get
							   to postscan
							 */
					sprintf(errmsg, "illegal option: %c", yytext[0]);
					cmderror(errmsg);
					bailout = TRUE;
				  }
				  return _OPT;
				}

<AS,PS,FS,OS,HS>all		{ return _ALL; }

<INITIAL>ALL			{ for (ii=strlen(ALL)-1; ii>=0; ii--)
					unput(ALL[ii]);
				}

<DS>{DL}[^ \t]*			{ RETURN(_DEFINE); }

<AS>{DL}[^ \t]*			{ RETURN(_ALERT); }

<PS>{DL}[^ \t]*			{ RETURN(_PROGRAM); }

<FS>{DL}[^ \t]*			{ RETURN(_FILE); }

<OS>{DL}[^ \t]*			{ RETURN(_OBJECT); }

<HS>{DL}[^ \t]*			{ RETURN(_HOST); }

<CS>\177[^\177]*\177		{ yytext[yyleng-1] = '\0';
				  cmdlval.sval = mystrdup(yytext+1, _cmdlex_);
				  if (!postscan) {
					if (hasmacro(cmdlval.sval))
						rcfbs |= (1L << MCRCFG);
				  }
				  return _COMMAND;
				}

<CS>("-"|"+")[A-Z]		{ BEGIN(INITIAL);
				  yyless(0);
				}

<CS>[^ \t]+			{ if (!postscan) {
					if (hasmacro(yytext))
						rcfbs |= (1L << MCRCFG);
				  }
				  RETURN(_COMMAND);
				}

<DS,CS,AS,PS,FS,OS,HS>{W}	{ ; }

<INITIAL,DS,CS,AS,PS,FS,OS,HS>.	{ return yytext[0]; }

%%

/* cmd error handler
 */
void
cmderror (const char *message)
{

	fprintf(stderr, "%s (in cmdparse()): %s\n", message, yytext ? yytext : "");
	fflush(stderr);
	bailout = TRUE;
	return;
}

/* feed command-line arguments to the lexer input stream
 */
signed int
cmdinput (void)
{
	char c;

	if (argvcurr > argvlast)
		return(EOF);
	if (stroffset == 0) {	/* at beginning of new argv */
		if (strchr(*argvcurr, ' ') || strchr(*argvcurr, '\t'))
			wrapstr(argvcurr);
	}
	if ((c = (*argvcurr)[stroffset++]) != '\0')
		return(c);
	/* end of argument, unwrap() if necessary, then move on to next */
	if (**argvcurr == '\177')
		unwrapstr(argvcurr);
	argvcurr++;
	stroffset = 0;
	return(' ');
}

/* put a wrapper around the given string to signal to the lexer that this
   is a multi-part command string
 */
void
wrapstr (char **s)
{
	char *t;

	t = (char *) mymalloc((strlen(*s)+3) * sizeof(char), "wrapstr");
	sprintf(t, "%c%s%c", '\177', *s, '\177');
	*s = t;
	return;
}

/* undo the wrapping in wrapstr(), necessary because we scan the
   command line twice and must put things back as they were
 */
void
unwrapstr (char **s)
{
	char *t;

	t = mystrdup((*s)+1, "unwrapstr");
	*(t+strlen(*s)-2) = '\0';
	*s = t;
	return;
}

/* determine if the given string has at least one macro embedded within
 */
int
hasmacro (char *s)
{
	register int i;
	register char c;
	register int escape;

	escape = FALSE;
	i = 0;
	while ((c = *(s+i++))) {
		if ((c == '\\') && !escape) {
			escape = TRUE;
			continue;
		} else if (c == '=')
			return TRUE;
		escape = FALSE;
	}
	return FALSE;
}
