/*
   piktc.h -- PIKT common header file for piktc & piktc_svc

   $Header: /home/cvs/ucgsb/pikt/src/piktc/piktc.h,v 1.10 1999/11/29 05:10:20 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define MODE_PRG_DEFAULT	"750"	/* default mode for installed
					   programs
					 */
#define MODE_FIL_DEFAULT	"640"	/* default mode for installed
					   files
					 */
#define UID_DEFAULT		"0"	/* default uid for installed
					   programs/files
					 */
#define GID_DEFAULT		"0"	/* default gid for installed
					   programs/files
					 */

/* additional rpc request states */
#define DELETE		0
#define ADD		1
#define EXECUTE		2
#define EXECUTEWAIT	3

/* the order of these is vitally important! */
#define OPS		0
#define HST		1
#define GRP		2
#define ALS		3
#define DEF		4
#define MCR		5
#define ALM		6
#define ALT		7
#define PRG		8
#define FIL		9
#define OBJ		10

/* the order of these is vitally important! */
#define SYSCFG		0
#define DEFCFG		1
#define MCRCFG		2
#define ALMCFG		3
#define ALTCFG		4
#define PRGCFG		5
#define FILCFG		6
#define OBJCFG		7

/* used to record command-line alert/program/file/object/host hames;
   alert/program/file/object names in cfg.c
 */
typedef struct _nam {
	char *name;
	char *aux;		/* only used by alerts/programs/files */
	char *mode;		/* only used by programs/files for mode */
	char *uid;		/* only used by programs/files for uid */
	char *gid;		/* only used by programs/files for gid */
	int state;		/* ADD/DELETE/ERROR/IGNORE */
	struct _nam *next;
} nam;

/* used when passing non-file items from piktc to piktc_svc
*/
typedef struct _itm {
	char *name;
	char *aux;
	int state;		/* ADD/DELETE/ERROR/IGNORE */
} itm;

/* used when passing files from piktc to piktc_svc
 */
typedef struct _fil {
	char *name;		/* filename */
	char *directory;	/* file directory */
	char *extension;	/* file extension, e.g., .alt */
	char *mode;		/* file permissions */
	char *uid;		/* file uid */
	char *gid;		/* file gid */
	char *text;		/* file contents as one long char string */
	int size;		/* file size in bytes */
} fil;

/* used when passing service requests registrations to piktc_svc
 */
typedef struct _req {
	char *hostname;		/* name of the remote piktc_svc host */
	char *request;		/* the requested service */
	int state;		/* the service state */
} req;

/* used to store lines from all .cfg files */
typedef struct _cfgln {
	int lineno;		/* line number */
	char *file;		/* line's cfg file */
	unsigned long *bs;	/* line's host bitset "tag" */
	char *line;		/* line text */
	int len;		/* line length */
	int domcr;		/* macro expand this line? */
	int doproc;		/* process this line? */
	struct _cfgln *next;
} cfgln;

typedef struct _def {
	char *component;	/* macro/alarm def component */
	struct _def *next;
} def;

/* note that not all fields are used for every ident */
typedef struct _ident {
	char *name;		/* ident name */
	int type;		/* DEF/OPS/HST/GRP/ALS/MCR/ALM/ALT/PRG/FIL/OBJ */
	char *alias;		/* only used by ALS type */
	unsigned long *bs;	/* ident's host/define bitset "tag" */
	def *definition;	/* list of ident defs */
	struct _ident *next;
} ident;

extern const char piktc_version[];
extern const char piktc_svc_version[];

extern int cfgerr;	/* signifies syntax error in any config file */
extern int stzerr;	/* signifies syntax error in current stanza */

extern nam *commands, *incmdcommands, *cmdcommands,
           *alerts, *cfgalerts, *incmdalerts, *excmdalerts, *cmdalerts,
           *programs, *cfgprograms, *incmdprograms, *excmdprograms, *cmdprograms,
           *files, *cfgfiles, *incmdfiles, *excmdfiles, *cmdfiles,
           *objects, *cfgobjects, *incmdobjects, *excmdobjects, *cmdobjects,
                     *cfghosts,   *incmdhosts,   *excmdhosts,   *cmdhosts;

extern int commandsall, alertsall, programsall, filesall, objectsall, hostsall;

extern FILE	*stripin,	*stripout,
		*tagin,		*tagout,
		*sysin,		*sysout,
		*cfgin,		*cfgout;

extern char *infile;		/* current input infile */
extern int fd[];		/* channel from striplex() to *parse() */
extern pid_t pid;		/* pid of current child; for striplex() */

extern int p2cfd[][], c2pfd[][];	/* ipc parent-to-child and
					   child-to-parent channels;
					   for coordinating striplex()
					   with *parse()
					 */

extern const char *cfgfilenames[];	/* systems.cfg/defines.cfg/macros.cfg/
					   alarms.cfg/alerts.cfg/programs.cfg/
					   files.cfg/objects.cfg
					 */
extern char *cfgfileprev;	/* name of previous config file */
extern char *cfgfilecurr;	/* name of current config file */

extern const char *cfgbof[];	/* signifies beginning of new config file */
extern char *cfp;

extern int cfgcurr;		/* current config file */
extern int cfglast;		/* last config file to be processed */
extern cfgln *configs[];	/* array of pointers to cfg file lines */
extern cfgln *clplast;		/* pointer to last cfg line */

extern ident *gidp;			/* points to active ident */
extern ident *idents[];			/* ident hash table */
extern const char *identypes[];		/* OPS/HST/GRP/ALS/DEF/MCR/ALM/ALT/PRG/FIL/OBJ */

extern unsigned long  cfbs;		/* actual config bitset */
extern unsigned long rcfbs;		/* required config bitset */
extern unsigned long mcfbs;		/* missing config bitset */

extern size_t hbslongs;			/* number of longs in a host bit-set */
extern size_t dbslongs;			/* number of longs in a define bit-set */

extern unsigned long *ghbs;		/* global host bitset */

extern unsigned long *hbsfull, *hbsempty; /* full and empty host bitsets */

extern unsigned long *hbscurr;		/* host bitset for the current
					     piktc command
					 */

extern unsigned long *gdbs;		/* global define bitset */
extern unsigned long *gubs;		/* global undefined bitset */

extern unsigned long *dbsfull, *dbsempty; /* full and empty define bitsets;
					     currently serve no purpose
					   */

extern unsigned long *dbscurr;		/* define bitset for the current
					   piktc command
					 */

extern char **hosts;			/* system hostnames */
extern int hostnum;			/* the number of configured hosts */
extern char *hostcurr;			/* the current hostname */

extern char **opsys;			/* operating systems */
extern int opsysnum;			/* the number of configured os'es */

extern int defnum;			/* the number of configured defines */

extern const char piktnullchar[];	/* for =piktnullchar definition */
extern const char piktspacechar[];	/* for =piktspacechar definition */
extern const char pikttabchar[];	/* for =pikttabchar definition */
extern const char piktnewlinechar[];	/* for =piktnewlinechar definition */

extern unsigned stroffset;		/* offset from beginning of string */
extern unsigned bof;			/* beginning of file */
extern unsigned bon;			/* beginning of (line) number */
extern unsigned bol;			/* beginning of line */

extern def *defcurr;			/* current definition */

extern char *mins, *hrs, *dom, *mons, *dow;	/* timing parameters */
extern char *priority;				/* alert nice level */
extern char *nicecmd, *mailcmd, *lpcmd;		/* strings written to
						   piktd.conf
						 */

extern int stripalm, stripprg, stripfil, stripobj;	/* strip these files
							   of comments?
							 */

extern int domacros;			/* do macro processing? */
extern int dostrip;			/* do comment stripping? */

extern int incnum;			/* the current (nested) include
					   file number
					 */

extern char *hostserv;			/* piktc_svc service host */
extern char *rqstserv;			/* call-back service request */
extern int statserv;			/* piktc_svc service state */

extern const char *transports[];	/* "udp" or "tcp" */

/* init functions */
extern int chkpiktc_svc(char *host);
extern int piktc_svc_main(void);

/* command-line functions */
extern void piktc_usage(void);
extern void piktc_svc_usage(void);
extern void preprocesscmd(void);
extern nam *regname(nam **, char *, int, int);
extern void retire(nam **);
extern nam *lookupname(nam *, char *);

/* lexer and parser functions */
extern int cmdparse(void);
extern int sysparse(void);
extern int tagparse(void);
extern int defparse(void);
extern int cfgparse(void);
extern int cmdlex(void);
extern int syslex(void);
extern int taglex(void);
extern int deflex(void);
extern int cfglex(void);
extern void cmderror(const char *);
extern void syserror(const char *);
extern void tagerror(const char *);
extern void deferror(const char *);
extern void cfgerror(const char *);
extern int striplex(void);

/* ident functions */
extern ident *lookupident(char *, int);
extern void regident(char *, int, int);
void cleanup(void);
extern void makepiktmacros(void);
extern char *getnicecmd(char *);

/* bitset functions */
extern void empty(unsigned long *, size_t);
extern void fill(unsigned long *, size_t);
extern unsigned long *bsunion(unsigned long *, unsigned long *, size_t);
extern unsigned long *bsintersection(unsigned long *, unsigned long *, size_t);
extern unsigned long *bscomplement(unsigned long *, size_t);
extern int bsmember(unsigned long *, unsigned long *, size_t);
extern int bssubset(unsigned long *, unsigned long *, size_t);
extern void bscopy(unsigned long *, unsigned long *, size_t);
extern unsigned long *bsdup(unsigned long *, size_t);
extern unsigned long *getbs(char *, int);
extern void tag(int, char *, unsigned long *, size_t, int, int, char *);
extern ident *insert(char *, int);

/* alt/prg/fil/obj file functions */
extern void regcomponent(char *);
extern void beginalarms(char *);
extern void writealarm(char *);
extern void endalarms(char *);
extern void beginfile(char *, char *, char *, char *, nam **);
extern void writefile(char *);
extern void endfile(char *);
extern void beginobjects(char *);
extern void writeobject(char *);
extern void endobjects(char *);
extern int conflinelen(char *);

/* rpc functions */
extern void enable_alerts(char *, nam *, int);
extern void disable_alerts(char *, nam *, int);
extern void execute_commands(char *, nam *, int);
extern void execute_alerts(char *, nam *, int);
extern void install_files(char *, nam *, char *, const char *, char *, char *, char *, int);
extern void fetch_files(char *, nam *, char *, const char *, int);
extern void delete_files(char *, nam *, char *, const char *, int);
extern void list_files(char *, nam *, char *, const char *, int);
extern void kill_daemon(char *, unsigned long, int);
extern void restart_daemon(char *, unsigned long, int);
extern void show_alerts_stata(char *, nam *, int);
extern int free_execcmdout(char *, int);
extern int register_request(char *, char *, int);
extern int call_back(char *, const char *, int);
extern int install_file(char *, char *, char *, const char *, char *, char *, char *, int);

/* debugging functions */
extern void dumpcfgs(unsigned long *);
