%{

/*
   expr.y -- PIKT script expression parser

   $Header: /home/cvs/ucgsb/pikt/src/piktd/expr.y,v 1.29 1999/12/07 12:55:51 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include "../lib/common.h"

#include <math.h>

#include "piktd.h"

char _exprparse_[] = "exprparse";

#define NUMERR		if (doeval) exprres = ERROR; \
			yyval.fval = ERROR

#define STRERR		if (doeval) exprres = ERROR; \
			yyval.sval = mystrdup(err, _exprparse_)

#define CHKIFNUMUB	if (ifnum >= MAXIFS) { \
				logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in exprparse(), %s, ifnum >= MAXIFS (%d)", ALMNAM, MAXIFS); \
			} \

#define CHKIFNUMLB	if (ifnum <= 0) { \
				logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in exprparse(), %s, ifnum <= 0", ALMNAM); \
			}

int ifnum = 0;
int doevals[MAXIFS+1];
int doeval;
int ifconds[MAXIFS+1];

int i;
char c;
char *cp;
char tmp[MAXLINE+1];
char *concat;
int pos;
char *sysres;
int dpls;
char fmt[16+1];
char rdlin[2][MAXLINE+1];
char *rdlinnow, *rdlinprc, *rdlintmp;
chrary *vap[MAXVAP];
int matches;
int iserrdef;

%}

%union {
	signed char *sval;
	int ival;
	long lval;
	double fval;
}

%token <sval> _LP _RP _LB _RB _QLB _CMA
%token <sval> _NOT _AND _OR
%token <sval> _ADD _SUBTRACT _MULTIPLY _DIVIDE _MODULUS _EXPONENTIATE
%token <sval> _BITAND _BITOR _BITEXOR _BITCOMP _BITLSHFT _BITRSHFT
%token <sval> _CONCATENATE
%token <sval> _EQ _GT _LT _GE _LE _NGT _NLT _NE _NE2 _RGXP _RGXPI _NRGXP _NRGXPI _SEQ _SNE _SLT _SGT _SLE _SGE
%token <sval> _EXIST _ZEROLEN _REGFILE _DIRECTORY _CHARSPEC _BLKSPEC _PIPE _SYMLINK _SOCKET

%token _QT
%token <ival> _ARYSTR _ARYNUM

%token <sval> _ABSVAL
%token <sval> _AGE
%token <sval> _ALARM
%token <sval> _AMPM
%token <sval> _ANDFN
%token <sval> _AVERAGE
%token <sval> _BASENAME
%token <sval> _CEILING
%token <sval> _CHAR
%token <sval> _CHECKSUM
%token <sval> _CHOP
%token <sval> _CODE
%token <sval> _COMMAND
%token <sval> _DATEVALUE
%token <sval> _DAY
%token <sval> _DAYNAME
%token <sval> _DAYNUMBER
%token <sval> _DAYSBETWEEN
%token <sval> _DEFINED
%token <sval> _DIRNAME
%token <sval> _DQUOTE
%token <sval> _ENDOFINPUT
%token <sval> _ERRFNNUM
%token <sval> _ERRFNSTR
%token <sval> _ERROR
%token <sval> _EVEN
%token <sval> _FALSEFN
%token <sval> _FILEMODE
%token <sval> _FILEINO
%token <sval> _FILELINKS
%token <sval> _FILEUID
%token <sval> _FILEGID
%token <sval> _FILESIZE
%token <sval> _FILEATIME
%token <sval> _FILEMTIME
%token <sval> _FILECTIME
%token <sval> _FIXED
%token <sval> _GRNAME
%token <sval> _GRPASSWORD
%token <sval> _GRGID
%token <sval> _GRMEMBERS
%token <sval> _HOSTNAME
%token <sval> _HOUR
%token <sval> _IFFNNUM
%token <sval> _IFFNSTR
%token <sval> _INPUTLINE
%token <sval> _INPUTLENGTH
%token <sval> _INPUTNUMBER
%token <sval> _INTEGER
%token <sval> _ISITALNUM
%token <sval> _ISITALPHA
%token <sval> _ISITBLANK
%token <sval> _ISITCNTRL
%token <sval> _ISITDIGIT
%token <sval> _ISITGRAPH
%token <sval> _ISITLOWER
%token <sval> _ISITPRINT
%token <sval> _ISITPUNCT
%token <sval> _ISITSPACE
%token <sval> _ISITUPPER
%token <sval> _LEFT
%token <sval> _LENGTH
%token <sval> _LOWER
%token <sval> _LTRIM
%token <sval> _MAXIMUM
%token <sval> _MEDIAN
%token <sval> _MINIMUM
%token <sval> _MINUTE
%token <sval> _MODE
%token <sval> _MONTH
%token <sval> _MONTHNAME
%token <sval> _MONTHNUMBER
%token <sval> _NEWLINE
%token <sval> _NILFNNUM
%token <sval> _NILFNSTR
%token <sval> _NOTFN
%token <sval> _NOW
%token <sval> _ODD
%token <sval> _ORFN
%token <sval> _PROCID
%token <sval> _PPROCID
%token <sval> _PROPER
%token <sval> _PWNAME
%token <sval> _PWPASSWORD
%token <sval> _PWUID
%token <sval> _PWGID
%token <sval> _PWGECOS
%token <sval> _PWDIR
%token <sval> _PWSHELL
%token <sval> _RANDOM
%token <sval> _REPEAT
%token <sval> _REPLACE
%token <sval> _REVERSE
%token <sval> _RIGHT
%token <sval> _ROUND
%token <sval> _RSEARCH
%token <sval> _RTRIM
%token <sval> _SEARCH
%token <sval> _SECOND
%token <sval> _SPACE
%token <sval> _SQUOTE
%token <sval> _SUBSTITUTE
%token <sval> _SUBSTRING
%token <sval> _SYSTEM
%token <sval> _TAB
%token <sval> _TIMEVALUE
%token <sval> _TODAY
%token <sval> _TRIM
%token <sval> _TRUEFN
%token <sval> _TRUNC
%token <sval> _UPPER
%token <sval> _VALUE
%token <sval> _WEEKDAY
%token <sval> _YEAR
%token <sval> _YEARDAY
%token <sval> _YEARWEEK

%token <sval> _FOPEN
%token <sval> _POPEN
%token <sval> _READ
%token <sval> _WRITE
%token <sval> _FCLOSE
%token <sval> _PCLOSE

%token <sval> _PARSE

%token <fval> _NUM
%token <ival> _INT
%token _NULSTR
%token <sval> _STR _ERR _NIL

%type <ival> conditions relation expression expfil andconds orconds
%type <fval> expnum number numfn str_as_num iffnnumend
%type <sval> expstr string strfn num_as_str tok_as_str iffnstrend

%left _OR
%left _AND
%left _BITOR _BITEXOR
%left _BITAND
%nonassoc _EQ _NE _NE2 _SEQ _SNE
%nonassoc _LT _GT _LE _GE _NLT _NGT _SLT _SGT _SLE _SGE
%nonassoc _RGXP _RGXPI _NRGXP _NRGXPI
%nonassoc _EXIST _ZEROLEN _REGFILE _DIRECTORY _CHARSPEC _BLKSPEC _PIPE _SYMLINK _SOCKET
%right _BITLSHFT _BITRSHFT
%left _ADD _SUBTRACT _CONCATENATE
%left _MULTIPLY _DIVIDE _MODULUS
%right _EXPONENTIATE
%nonassoc _NOT _BITCOMP _UMINUS

%%

start:		{
			doeval = TRUE;
			iserrdef = FALSE;
			vapndx = 0;
		}
		conditions {
			switch ($2) {
				case TRUE:
					exprres = TRUE;
					break;
				case FALSE:
					exprres = FALSE;
					break;
				case ERROR:
					exprres = FALSE;
					stmcurr->status = INACTIVE;
					break;
				case NIL:
					exprres = FALSE;
					stmcurr->status = INACTIVE;
					break;
			}
		}
		;

conditions:	expression {
			$$ = $1;
		}
		| relation {
			$$ = $1;
			exprstr = (char *) mymalloc(MAXINTSTR+1, _exprparse_);
			sprintf(exprstr, "%d", $$);
		}
		| _LB conditions _RP {
			$$ = $2;
			exprstr = (char *) mymalloc(MAXINTSTR+1, _exprparse_);
			sprintf(exprstr, "%d", $$);
			if (isdebug)
				logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "( %d ) = %d", $2, $$);
			free($1);
			free($3);
		}
		| _NOT conditions {
			switch ($2) {
				case TRUE:
					$$ = FALSE;
					break;
				case FALSE:
					$$ = TRUE;
					break;
				case ERROR:
					$$ = ERROR;
					break;
				case NIL:
					$$ = NIL;
					break;
			}
			exprstr = (char *) mymalloc(MAXINTSTR+1, _exprparse_);
			sprintf(exprstr, "%d", $$);
			if (isdebug)
				logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "! %d = %d", $2, $$);
			free($1);
		}
		| conditions
		_AND {
			switch ($1) {
				case TRUE:
					doeval = TRUE;
					break;
				case FALSE:
				case ERROR:
				case NIL:
					doeval = FALSE;
					break;
			}
		}
		conditions {
			switch ($1) {
				case TRUE:
					switch ($4) {
						case TRUE:
							$$ = TRUE;
							break;
						case FALSE:
							$$ = FALSE;
							break;
						case ERROR:
							$$ = ERROR;
							break;
						case NIL:
							$$ = NIL;
							break;
					}
					break;
				case FALSE:
					switch ($4) {
						case TRUE:
							$$ = FALSE;
							break;
						case FALSE:
							$$ = FALSE;
							break;
						case ERROR:
							$$ = FALSE;
							break;
						case NIL:
							$$ = FALSE;
							break;
					}
					break;
				case ERROR:
					switch ($4) {
						case TRUE:
							$$ = ERROR;
							break;
						case FALSE:
							$$ = FALSE;
							break;
						case ERROR:
							$$ = ERROR;
							break;
						case NIL:
							$$ = ERROR;
							break;
					}
					break;
				case NIL:
					switch ($4) {
						case TRUE:
							$$ = NIL;
							break;
						case FALSE:
							$$ = FALSE;
							break;
						case ERROR:
							$$ = ERROR;
							break;
						case NIL:
							$$ = NIL;
							break;
					}
					break;
			}
			exprstr = (char *) mymalloc(MAXINTSTR+1, _exprparse_);
			sprintf(exprstr, "%d", $$);
			if (isdebug)
				logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%d && %d = %d", $1, $4, $$);
			doeval = TRUE;
			free($2);
		}
		| conditions
		_OR {
			switch ($1) {
				case TRUE:
					doeval = FALSE;
					break;
				case FALSE:
				case ERROR:
				case NIL:
					doeval = TRUE;
					break;
			}
		}
		conditions {
			switch ($1) {
				case TRUE:
					switch ($4) {
						case TRUE:
							$$ = TRUE;
							break;
						case FALSE:
							$$ = TRUE;
							break;
						case ERROR:
							$$ = TRUE;
							break;
						case NIL:
							$$ = TRUE;
							break;
					}
					break;
				case FALSE:
					switch ($4) {
						case TRUE:
							$$ = TRUE;
							break;
						case FALSE:
							$$ = FALSE;
							break;
						case ERROR:
							$$ = ERROR;
							break;
						case NIL:
							$$ = NIL;
							break;
					}
					break;
				case ERROR:
					switch ($4) {
						case TRUE:
							$$ = TRUE;
							break;
						case FALSE:
							$$ = ERROR;
							break;
						case ERROR:
							$$ = ERROR;
							break;
						case NIL:
							$$ = ERROR;
							break;
					}
					break;
				case NIL:
					switch ($4) {
						case TRUE:
							$$ = TRUE;
							break;
						case FALSE:
							$$ = NIL;
							break;
						case ERROR:
							$$ = ERROR;
							break;
						case NIL:
							$$ = NIL;
							break;
					}
					break;
			}
			exprstr = (char *) mymalloc(MAXINTSTR+1, _exprparse_);
			sprintf(exprstr, "%d", $$);
			if (isdebug)
				logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%d || %d = %d", $1, $4, $$);
			doeval = TRUE;
			free($2);
		}
		;

expression:	expnum {
			if (doeval) {
				$$ = ($1 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f ? %d", $1, $$);
				exprval = $1;
				exprstr = (char *) mymalloc(MAXDBLSTR+1, _exprparse_);
				sprintf(exprstr, "%f", $1);
			} else
				$$ = NIL;
		}
		| expstr {
			if (doeval) {
				$$ = (strlen($1) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s ? %d", $1, $$);
				exprstr = $1;
				errno = 0;
				exprval = atof($1);
				/*
				if (errno)
					$$ = ERROR;
				 */
			} else
				$$ = NIL;
		}
		| expfil {
			if (doeval) {
				$$ = ($1 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%d ? %d", $1, $$);
				exprval = $1;
				exprstr = (char *) mymalloc(MAXINTSTR+1, _exprparse_);
				sprintf(exprstr, "%d", $1);
			} else
				$$ = NIL;
		}
		;

relation:	expnum _EQ expnum {
			if (doeval) {
				$$ = ((float)$1 == (float)$3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f == %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _GT expnum {
			if (doeval) {
				$$ = ($1 > $3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f > %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _LT expnum {
			if (doeval) {
				$$ = ($1 < $3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f < %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _GE expnum {
			if (doeval) {
				$$ = ($1 >= $3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f >= %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _LE expnum {
			if (doeval) {
				$$ = ($1 <= $3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f <= %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _NGT expnum {
			if (doeval) {
				$$ = (!($1 > $3) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "!(%f > %f) ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _NLT expnum {
			if (doeval) {
				$$ = (!($1 < $3) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "!(%f < %f) ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _NE expnum {
			if (doeval) {
				$$ = ((float)$1 != (float)$3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f != %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _NE2 expnum {
			if (doeval) {
				$$ = ((float)$1 != (float)$3 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%f != %f ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($2);
		}
		| expstr _SEQ expstr {
			if (doeval) {
				$$ = (!strcmp($1,$3) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "@%s@ eq @%s@ ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| expstr _SNE expstr {
			if (doeval) {
				$$ = (strcmp($1,$3) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s ne %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		|
		expstr _SLT expstr {
			if (doeval) {
				$$ = (strcmp($1,$3) < 0 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s lt %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		|
		expstr _SGT expstr {
			if (doeval) {
				$$ = (strcmp($1,$3) > 0 ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s gt %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		|
		expstr _SLE expstr {
			if (doeval) {
				$$ = ((strcmp($1,$3) < 0) || (strcmp($1,$3) == 0) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s le %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		|
		expstr _SGE expstr {
			if (doeval) {
				$$ = ((strcmp($1,$3) > 0) || (strcmp($1,$3) == 0) ? TRUE : FALSE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s ge %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| expstr _RGXP expstr {
			if (doeval) {
				$$ = regexp($1,$3,REG_EXTENDED,FALSE) ? TRUE : FALSE;
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s =~ %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| expstr _RGXPI expstr {
			if (doeval) {
				$$ = regexp($1,$3,REG_EXTENDED|REG_ICASE,FALSE) ? TRUE : FALSE;
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s =~ %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| expstr _NRGXP expstr {
			if (doeval) {
				$$ = !regexp($1,$3,REG_EXTENDED,FALSE) ? TRUE : FALSE;
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s !~ %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| expstr _NRGXPI expstr {
			if (doeval) {
				$$ = !regexp($1,$3,REG_EXTENDED|REG_ICASE,FALSE) ? TRUE : FALSE;
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s !~ %s ? %d", $1, $3, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		;

expnum:		number {
			if (doeval)
				$$ = $1;
			else
				$$ = NIL;
		}
		| str_as_num {
			if (doeval)
				$$ = $1;
			else
				$$ = NIL;
		}
		| numfn { $$ = $1; }
		| _LP expnum _RP {
			if (doeval)
				$$ = $2;
			else
				$$ = NIL;
			free($1);
			free($3);
		}
		| expnum _ADD expnum {
			if (doeval)
				$$ = $1 + $3;
			else
				$$ = NIL;
			free($2);
		}
		| expnum _SUBTRACT expnum {
			if (doeval)
				$$ = $1 - $3;
			else
				$$ = NIL;
			free($2);
		}
		| expnum _MULTIPLY expnum {
			if (doeval)
				$$ = $1 * $3;
			else
				$$ = NIL;
			free($2);
		}
		| expnum _DIVIDE expnum {
			if (doeval) {
				if ($3)
				  	$$ = $1 / $3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted divide by zero: %f / %f", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _MODULUS expnum {
			if (doeval) {
				if ((int)$3)
				  	$$ = (long)$1 % (long)$3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted modulo zero: %f %% %f", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _EXPONENTIATE expnum {
			if (doeval) {
				if ((int)$3)
				  	$$ = pow($1,$3);
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted exponentiation with non-integer power: %f ** %f", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _BITAND expnum {
			if (doeval) {
				if (isint($1) && isint($3))
					$$ = (long)$1 & (long)$3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted bitwise _AND of nonintegral or negative operand(s): %f & %f ", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _BITOR expnum {
			if (doeval) {
				if (isint($1) && isint($3))
					$$ = (long)$1 | (long)$3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted bitwise inclusive _OR of nonintegral operand(s): %f | %f ", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _BITEXOR expnum {
			if (doeval) {
				if (isint($1) && isint($3))
					$$ = (long)$1 ^ (long)$3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted bitwise exclusive _OR of nonintegral operand(s): %f ^ %f ", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _BITLSHFT expnum {
			if (doeval) {
				if (isint($1) && isint($3))
					$$ = (long)$1 << (long)$3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted left shift of nonintegral operand(s): %f << %f ", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;

				}
			} else
				$$ = NIL;
			free($2);
		}
		| expnum _BITRSHFT expnum {
			if (doeval) {
				if (isint($1) && isint($3))
					$$ = (long)$1 >> (long)$3;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted right shift of nonintegral operand(s): %f << %f ", ALMNAM, $1, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
		}
		| _SUBTRACT expnum %prec _UMINUS {
			if (doeval)
				$$ = -$2;
			else
				$$ = NIL;
			free($1);
		}
		| _BITCOMP expnum {
			if (doeval) {
				if (isint($2))
					$$ = ~ (long)$2;
				else {
				  	logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, attempted one's complement of nonintegral operand: ~ %f ", ALMNAM, $2);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
		}
		;

expstr:		_QT string _QT { $$ = $2; }
		| strfn { $$ = $1; }
		| expstr _CONCATENATE expstr {
			if (doeval) {
				concat = (char *) mymalloc(strlen($1)+strlen($3)+1, _exprparse_);
				strcpy(concat, $1);
				strcat(concat, $3);
				$$ = concat;
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		;

expfil:		_EXIST expstr {
			if (doeval) {
				$$ = filestat($2, EXIST);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-e %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _ZEROLEN expstr {
			if (doeval) {
				$$ = filestat($2, ZEROLEN);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-z %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _REGFILE expstr {
			if (doeval) {
				$$ = filestat($2, REGFILE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-f %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _DIRECTORY expstr {
			if (doeval) {
				$$ = filestat($2, DIRECTORY);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-d %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _CHARSPEC expstr {
			if (doeval) {
				$$ = filestat($2, CHARSPEC);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-c %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _BLKSPEC expstr {
			if (doeval) {
				$$ = filestat($2, BLKSPEC);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-b %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _PIPE expstr {
			if (doeval) {
				$$ = filestat($2, PIPE);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-p %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _SYMLINK expstr {
			if (doeval) {
				$$ = filestat($2, SYMLINK);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-l %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		| _SOCKET expstr {
			if (doeval) {
				$$ = filestat($2, SOCKET);
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "-S %s ? %d", $2, $$);
			} else
				$$ = NIL;
			free($1);
			free($2);
		}
		;

number:		_NUM {
			if (doeval)
				$$ = $1;
			else
				$$ = NIL;
		}
		| _INT {
			if (doeval)
				$$ = (double)$1;
			else
				$$ = NIL;
		}
		| _ARYNUM _LB expnum _RB {
			--vapndx;
			if (doeval) {
				if (isint($3) && ($3 > 0) && ($3 <= MAXARYNDX)) {
					if (vap[vapndx] && vap[vapndx]->subary[SA((unsigned short)$3)] && vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)]) {
						strcpy(tmp, vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)]);
						if ((cp = strchr(tmp,'/'))) {
							errno = 0;
							*cp = '\0';
							$$ = atof(tmp)/atof(cp+1);
						} else if ((cp = strchr(tmp,'%'))) {
							errno = 0;
							*cp = '\0';
							$$ = atof(tmp)/100;
						} else {
							errno = 0;
							$$ = atof(tmp);
						}
						if (errno) {
							logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s, var #%s[%d] is not a number", ALMNAM, VARNAM($1), (unsigned short)$3);
							NUMERR;
							$$ = ERROR;
						}
						vap[vapndx] = (chrary *) NULL;
					} else {
						logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, attempt to reference nonexistent value for var #%s[%d]", ALMNAM, VARNAM($1), (int)$3);
						NUMERR;
						$$ = ERROR;
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, for var #%s[], non-integral array index: %f", ALMNAM, VARNAM($1), $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($2);
			free($4);
		}
		;

string:		_STR {
			if (doeval)
				$$ = $1;
			else {
				$$ = mystrdup(nil, _exprparse_);
				free($1);
			}
		}
		| _ERR {
			if (doeval)
				$$ = mystrdup(errstr, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
		}
		| _NIL {
			if (doeval)
				$$ = mystrdup(nilstr, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
		}
		| _NULSTR {
			if (doeval) {
				$$ = mystrdup("", _exprparse_);
			} else
				$$ = mystrdup(nil, _exprparse_);
		}
		| _ARYSTR _LB expnum _RB {
			--vapndx;
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXARYNDX)) {
					if (vap[vapndx]) {
						if (vap[vapndx]->subary[SA((unsigned short)$3)] && vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)] && ($3 > 0)) {
							$$ = mystrdup(vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)], _exprparse_);
							vap[vapndx] = (chrary *) NULL;
						} else {
							logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, attempt to reference nonexistent value for var $%s[%d]", ALMNAM, VARNAM($1), (unsigned short)$3);
							STRERR;
							$$ = mystrdup(err, _exprparse_);
						}
					} else {
						if (valscn[(unsigned short)$3]) {
							$$ = mystrdup(valscn[(unsigned short)$3], _exprparse_);
						} else {
							logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, attempt to reference nonexistent value $[%d]", ALMNAM, (int)$3);
							STRERR;
							$$ = mystrdup(err, _exprparse_);
						}
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, for var $%s[], non-integral array index: %f", ALMNAM, VARNAM($1), $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($2);
			free($4);
		}
		| num_as_str {
			if (doeval)
				$$ = $1;
			else {
				$$ = mystrdup(nil, _exprparse_);
				free($1);
			}
		}
		| tok_as_str {
			if (doeval)
				$$ = $1;
			else {
				$$ = mystrdup(nil, _exprparse_);
				free($1);
			}
		}
		;

statexprs:	expnum {
			regstat($1);
		}
		| statexprs _CMA expnum {
			regstat($3);
			free($2);
		}
		;

andconds:	conditions {
			$$ = $1 ? TRUE : FALSE;
		}
		| andconds _CMA conditions {
			$$ = $1 && $3 ? TRUE : FALSE;
			free($2);
		}
		;

orconds:	conditions {
			$$ = $1 ? TRUE : FALSE;
		}
		| orconds _CMA conditions {
			$$ = $1 || $3 ? TRUE : FALSE;
			free($2);
		}
		;

numfn:		_DEFINED _LP _QT string _QT _RP {
			if (doeval) {
				if (!strcmp($4,nilstr))
					$$ = FALSE;
				else if (!strcmp($4,errstr))
					$$ = FALSE;
				else
					$$ = TRUE;
			} else
				$$ = NIL;
			iserrdef = FALSE;
			free($1);
			free($2);
			free($4);
			free($6);
		}
		| _DEFINED _LP number _RP {
			if (doeval) {
				$$ = TRUE;
			} else
				$$ = NIL;
			iserrdef = FALSE;
			free($1);
			free($2);
			free($4);
		}
		| _DEFINED _LP _ERR _RP {
			if (doeval)
				$$ = FALSE;
			else
				$$ = NIL;
			iserrdef = FALSE;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _DEFINED _LP _NIL _RP {
			if (doeval)
				$$ = FALSE;
			else
				$$ = NIL;
			iserrdef = FALSE;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ERROR _LP expstr _RP {
			if (doeval) {
				if (!strcmp($3,err))
					$$ = TRUE;
				else
					$$ = FALSE;
			} else
				$$ = NIL;
			iserrdef = FALSE;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ERROR _LP expnum _RP {
			if (doeval) {
				if ($3 == ERROR)
					$$ = TRUE;
				else
					$$ = FALSE;
			} else
				$$ = NIL;
			iserrdef = FALSE;
			free($1);
			free($2);
			free($4);
		}
		| _TRUEFN _LP _RP {
			if (doeval)
				$$ = TRUE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _FALSEFN _LP _RP {
			if (doeval)
				$$ = FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _ANDFN _LP andconds _RP {
			if (doeval)
				$$ = $3;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _ORFN _LP orconds _RP {
			if (doeval)
				$$ = $3;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _NOTFN _LP conditions _RP {
			if (doeval)
				$$ = $3 ? FALSE : TRUE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _IFFNNUM _LP conditions {
			CHKIFNUMUB;
			ifnum++;
			doevals[ifnum] = doeval;
			if ((ifconds[ifnum] = $3))
				doeval = doevals[ifnum];
			else
				doeval = FALSE;
		} _CMA expnum iffnnumend {
			doeval = doevals[ifnum];
			CHKIFNUMLB;
			ifnum--;
			if (doeval)
				$$ = $3 ? $6 : $7;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($5);
		}
		| _ENDOFINPUT _LP _RP {
			if (doeval)
				$$ = eoi;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _EVEN _LP expnum _RP {
			if (doeval) {
				if (!isint($3)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in even(), %s, non-integral operand: %f", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				} else
				$$ = even((int)$3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _ODD _LP expnum _RP {
			if (doeval) {
				if (!isint($3)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in odd(), %s, non-integral operand: %f", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				} else
				$$ = odd((int)$3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _ERRFNNUM _LP _RP {
			if (doeval)
				$$ = ERROR;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _FILEINO _LP expstr _RP {
			if (doeval) {
				$$ = fileino($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILELINKS _LP expstr _RP {
			if (doeval) {
				$$ = filelinks($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILEUID _LP expstr _RP {
			if (doeval) {
				$$ = fileuid($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILEGID _LP expstr _RP {
			if (doeval) {
				$$ = filegid($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILESIZE _LP expstr _RP {
			if (doeval) {
				$$ = filesize($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILEATIME _LP expstr _RP {
			if (doeval) {
				$$ = fileatime($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILECTIME _LP expstr _RP {
			if (doeval) {
				$$ = filectime($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FILEMTIME _LP expstr _RP {
			if (doeval) {
				$$ = filemtime($3);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITALNUM _LP expstr _RP {
			if (doeval)
				$$ = isalnum((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _GRGID _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXGID))
					$$ = grgid((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in grgid(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _GRGID _LP expstr _RP {
			if (doeval)
				$$ = grgid(NIL, $3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWUID _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwuid((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwuid(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _PWUID _LP expstr _RP {
			if (doeval)
				$$ = pwuid(NIL, $3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWGID _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwgid((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwgid(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _PWGID _LP expstr _RP {
			if (doeval)
				$$ = pwgid(NIL, $3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITALPHA _LP expstr _RP {
			if (doeval)
				$$ = isalpha((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITBLANK _LP expstr _RP {
			if (doeval)
				$$ = (($3[0] == ' ') || ($3[0] == '\t')) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITCNTRL _LP expstr _RP {
			if (doeval)
				$$ = iscntrl((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITDIGIT _LP expstr _RP {
			if (doeval)
				$$ = isdigit((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITGRAPH _LP expstr _RP {
			if (doeval)
				$$ = isgraph((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITLOWER _LP expstr _RP {
			if (doeval)
				$$ = islower((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITPRINT _LP expstr _RP {
			if (doeval)
				$$ = isprint((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITPUNCT _LP expstr _RP {
			if (doeval)
				$$ = ispunct((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITSPACE _LP expstr _RP {
			if (doeval)
				$$ = isspace((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ISITUPPER _LP expstr _RP {
			if (doeval)
				$$ = isupper((int)$3[0]) ? TRUE : FALSE;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ABSVAL _LP expnum _RP {
			if (doeval)
				$$ = $3 >= 0 ? $3 : -$3;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _VALUE _LP expstr _RP {
			if (doeval) {
				errno = 0;
				$$ = atof($3);
				if (errno) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, TRUE, "in value(), %s, can't resolve string %s to number", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _VALUE _LP expnum _RP {
			if (doeval) {
				$$ = $3;
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _AVERAGE _LP {
			numcnt = 0;
			numsum = 0.0;
		} statexprs _RP {
			if (doeval)
				$$ = numsum/numcnt;
			else
				$$ = NIL;
			freestats();
			free($1);
			free($2);
			free($5);
		}
		| _MAXIMUM _LP {
			numcnt = 0;
		} statexprs _RP {
			if (doeval)
				$$ = nummax;
			else
				$$ = NIL;
			freestats();
			free($1);
			free($2);
			free($5);
		}
		| _MINIMUM _LP {
			numcnt = 0;
		} statexprs _RP {
			if (doeval)
				$$ = nummin;
			else
				$$ = NIL;
			freestats();
			free($1);
			free($2);
			free($5);
		}
		| _MEDIAN _LP {
			numcnt = 0;
		} statexprs _RP {
			if (doeval)
				$$ = median();
			else
				$$ = NIL;
			freestats();
			free($1);
			free($2);
			free($5);
		}
		| _MODE _LP statexprs _RP {
			if (doeval)
				$$ = mode();
			else
				$$ = NIL;
			freestats();
			free($1);
			free($2);
			free($4);
		}
		| _NILFNNUM _LP _RP {
			if (doeval)
				$$ = NIL;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _NOW _LP _RP {
			if (doeval)
				$$ = now();
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _TODAY _LP _RP {
			if (doeval)
				$$ = today();
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _DATEVALUE _LP expnum _CMA expnum _CMA expnum _RP {
			if (doeval)
				$$ = datevalue((int)$3,(int)$5,(int)$7);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($6);
			free($8);
		}
		| _TIMEVALUE _LP expnum _CMA expnum _CMA expnum _RP {
			if (doeval)
				$$ = timevalue((int)$3,(int)$5,(int)$7);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($6);
			free($8);
		}
		| _SECOND _LP expnum _RP {
			if (doeval)
				$$ = second((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _SECOND _LP _RP {
			if (doeval)
				$$ = second(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _MINUTE _LP expnum _RP {
			if (doeval)
				$$ = minute((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _MINUTE _LP _RP {
			if (doeval)
				$$ = minute(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _HOUR _LP expnum _RP {
			if (doeval)
				$$ = hour((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _HOUR _LP _RP {
			if (doeval)
				$$ = hour(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _DAY _LP expnum _RP {
			if (doeval)
				$$ = day((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _DAY _LP _RP {
			if (doeval)
				$$ = day(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _MONTH _LP expnum _RP {
			if (doeval)
				$$ = month((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _MONTH _LP _RP {
			if (doeval)
				$$ = month(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _YEAR _LP expnum _RP {
			if (doeval)
				$$ = year((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _YEAR _LP _RP {
			if (doeval)
				$$ = year(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _WEEKDAY _LP expnum _RP {
			if (doeval)
				$$ = weekday((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _WEEKDAY _LP _RP {
			if (doeval)
				$$ = weekday(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _YEARDAY _LP expnum _RP {
			if (doeval)
				$$ = yearday((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _YEARDAY _LP _RP {
			if (doeval)
				$$ = yearday(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _YEARWEEK _LP expnum _RP {
			if (doeval)
				$$ = yearweek((time_t)$3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _YEARWEEK _LP _RP {
			if (doeval)
				$$ = yearweek(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _MONTHNUMBER _LP expstr _RP {
			if (doeval)
				$$ = monthnumber($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _MONTHNUMBER _LP _RP {
			if (doeval)
				$$ = month(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _DAYNUMBER _LP expstr _RP {
			if (doeval)
				$$ = daynumber($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _DAYNUMBER _LP _RP {
			if (doeval)
				$$ = day(now());
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _DAYSBETWEEN _LP expnum _CMA expnum _RP {
			if (doeval)
				$$ = daysbetween((time_t)$3, (time_t)$5);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($6);
		}
		| _AGE _LP expstr _CMA expstr _CMA expstr _RP {
			if (doeval)
				$$ = age($3,$5,$7);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
			free($7);
			free($8);
		}
		| _CODE _LP expstr _RP {
			if (doeval)
				$$ = code($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _SEARCH _LP expstr _CMA expstr _CMA expnum _RP {
			if (doeval) {
				if ((($$ = search($3,$5,(int)$7)) == ERROR) || !isint($7)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in search(), %s, improper position operand: %f", ALMNAM, $7);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
			free($8);
		}
		| _SEARCH _LP expstr _CMA expstr _RP {
			if (doeval)
				$$ = search($3,$5,1);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
		}
		| _RSEARCH _LP expstr _CMA expstr _RP {
			if (doeval)
				$$ = rsearch($3,$5);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
		}
		| _LENGTH _LP expstr _RP {
			if (doeval)
				$$ = strlen($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _CEILING _LP expnum _RP {
			if (doeval)
				$$ = ceiling($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _INTEGER _LP expnum _RP {
			if (doeval)
				$$ = integer($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _ROUND _LP expnum _RP {
			if (doeval)
				$$ = round($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _TRUNC _LP expnum _RP {
			if (doeval)
				$$ = my_trunc($3);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _INPUTLENGTH _LP _RP {
			if (doeval)
				$$ = inlen;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _INPUTNUMBER _LP _RP {
			if (doeval)
				$$ = innum;
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _PROCID _LP expstr _RP {
			if (doeval)
				$$ = procid($3, PROCID);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PPROCID _LP expstr _RP {
			if (doeval)
				$$ = procid($3, PPROCID);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _RANDOM _LP _RP {
			if (doeval)
				$$ = myrandom(-1);
			else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
		}
		| _RANDOM _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0)) {
					$$ = myrandom($3);
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in random(), %s, non-integral or negative operand: %f", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _SYSTEM _LP expstr _RP {
			if (doeval) {
				if (exprres == ERROR) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in system(), %s, system() call aborted, error in command: %s", ALMNAM, $3);
					NUMERR;
					$$ = ERROR;
				} else {
					sysres = sys($3,SYSTEM);
					if (!strcmp(sysres, err)) {
						/* pikt log error entry done in sys() */
						NUMERR;
						$$ = ERROR;
					} else {
						errno = 0;
						$$ = atof(sysres);
						if (errno) {
							logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in system(), %s, bad exit value for command: %s", ALMNAM, $3);
							NUMERR;
							$$ = ERROR;
						} 
					free(sysres);
					}
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _FOPEN _LP _INT _CMA expstr _CMA expstr _RP {
			if (doeval) {
				if ((strlen($7) != 1) || !strchr("rwa",$7[0])) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in fopen(), %s, improper mode type: %s", ALMNAM, $7);
					NUMERR;
					$$ = ERROR;
				} else {
					if ((FLHNDL($3) = fopen($5,$7)) == NULL) {
						logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, TRUE, "in %s, fopen() error for %s", ALMNAM, $5);
						NUMERR;
						$$ = ERROR;
					} else {
						DATTYP($3) = FILEHANDLE;
						*rdlin[0] = *rdlin[1] = '\0';
						rdlinnow = rdlinprc = (char *) NULL;
						$$ = TRUE;
					}
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($5);
			free($6);
			free($7);
			free($8);
		}
		| _FCLOSE _LP _INT _RP {
			if (doeval) {
				if (fclose(FLHNDL($3))) {	/* returns 0 on success */
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s, fclose() error", ALMNAM);
					NUMERR;
					$$ = ERROR;
				} else {
					rdlinnow = rdlinprc = (char *) NULL;
					$$ = TRUE;
				}
				FLHNDL($3) = (FILE *) NULL;
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _POPEN _LP _INT _CMA expstr _CMA expstr _RP {
			if (doeval) {
				if ((strlen($7) != 1) || !strchr("rw",$7[0])) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in popen(), %s, improper mode type: %s", ALMNAM, $7);
					NUMERR;
					$$ = ERROR;
				} else {
					if ((FLHNDL($3) = popen($5,$7)) == NULL) {
						logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, TRUE, "in %s, popen() error for %s", ALMNAM, $5);
						NUMERR;
						$$ = ERROR;
					} else {
						DATTYP($3) = PROCHANDLE;
						*rdlin[0] = *rdlin[1] = '\0';
						rdlinnow = rdlinprc = (char *) NULL;
						$$ = TRUE;
					}
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($5);
			free($6);
			free($7);
			free($8);
		}
		| _PCLOSE _LP _INT _RP {
			if (doeval) {
				/*
				errno = 0;
				if (pclose(FLHNDL($3)) == -1) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s, pclose() error for file handle %s", ALMNAM, VARNAM($3));
					NUMERR;
					$$ = ERROR;
				} else {
					rdlinnow = rdlinprc = (char *) NULL;
					$$ = TRUE;
				}
				*/
				pclose(FLHNDL($3));
				rdlinnow = rdlinprc = (char *) NULL;
				$$ = TRUE;
				FLHNDL($3) = (FILE *) NULL;
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _READ _LP _INT _RP {
			if (doeval) {
				if (FLHNDL($3) == NULL) {
						logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in read(), %s, file not open for file handle %s", ALMNAM, VARNAM($3));
						NUMERR;
						$$ = ERROR;
				} else {
					if (!rdlinnow)
						rdlinnow = rdlin[0];
					else {
						if (rdlinprc) {
							rdlintmp = rdlinprc;
							rdlinprc = rdlinnow;
							rdlinnow = rdlintmp;
						} else {
							rdlinprc = rdlin[0];
							rdlinnow = rdlin[1];
						}
					}
					errno = 0;
					if (fgets(rdlinnow, MAXLINE, FLHNDL($3)) == NULL) {
						if (ferror(FLHNDL($3)) && (DATTYP($3) == FILEHANDLE)) {
							logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s, read() error for file handle %s", ALMNAM, VARNAM($3));
							clearerr(FLHNDL($3));
							NUMERR;
							$$ = ERROR;
						} else
							$$ = 0;
					} else {
						pos = dequote(rdlinnow);
						if (*(rdlinnow+pos-1) == '\n') {
							*(rdlinnow+pos-1) = '\0';
							/* pos--; */
						}
						$$ = pos;
					}
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _WRITE _LP _INT _CMA expstr _RP {
			if (doeval) {
				if (FLHNDL($3) == NULL) {
						logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in write(), %s, file not open for file handle %s", ALMNAM, VARNAM($3));
						NUMERR;
						$$ = ERROR;
				} else {
					requote($5);
					if (($$ = fprintf(FLHNDL($3),"%s\n",$5)) < 0) {
						logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s, write() error for string: %s", ALMNAM, $5);
						NUMERR;
						$$ = ERROR;
					}
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($5);
			free($6);
		}
		| _PARSE _LP expstr _RP {
			if (doeval) {
				freevalscn();
				valscn[0] = $3;
				$$ = matches = scanfields($3, " \t", TRUE);
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
		}
		| _PARSE _LP expstr _CMA expstr _RP {
			if (doeval) {
				if (regexp($3,$5,REG_EXTENDED,TRUE) != NIL)
					$$ = matches;
				else {
					freevalscn();
					valscn[0] = $3;
					$$ = matches = scanfields($3, $5, TRUE);
				}
			} else
				$$ = NIL;
			free($1);
			free($2);
			free($4);
			free($5);
			free($6);
		}
		;

strfn:		_ALARM _LP _RP {
			if (doeval)
				$$ = mystrdup(ALMNAM, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _AMPM _LP expnum _RP {
			if (doeval)
				$$ = mystrdup(ampm((time_t)$3), _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _AMPM _LP _RP {
			if (doeval)
				$$ = mystrdup(ampm(now()), _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _CHAR _LP expnum _RP {
			if (doeval) {
				if (isint($3)) {
					if (($3 >= 0) && ($3 <= 127))
						$$ = my_char((int)$3);
					else {
						logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in char(), %s, operand out-of-bounds: %f", ALMNAM, $3);
						STRERR;
						$$ = mystrdup(err, _exprparse_);
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in char(), %s, non-integral operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _CHECKSUM _LP expnum _CMA expstr _RP {
			if (doeval) {
				if (isint($3) && ($3 >= MINCHKSUM) && ($3 <= MAXCHKSUM)) {
					if (filestat($5, REGFILE) || filestat($5, SYMLINK) ) {
						sysres = checksum($3, $5);
						if (!strcmp(sysres, err)) {
							/* pikt log error entry done in sys() */
							free(sysres);
							STRERR;
							$$ = mystrdup(err, _exprparse_);
						} else {
							$$ = sysres;
						}
					} else {
						logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in checksum(), %s, file nonexistent, else is not a regular file or symlink: %s", ALMNAM, $5);
						STRERR;
						$$ = mystrdup(err, _exprparse_);
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in checksum(), %s, non-integral or invalid numerical operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
			free($5);
			free($6);
		}
		| _CHOP _LP expstr _CMA expnum _RP {
			if (doeval) {
				if (isint($5) && ($5 >= 0)) {
					$$ = chop($3, (int)$5);
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in chop(), %s, non-integral or negative decimal operand: %f", ALMNAM, $5);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($6);
		}
		| _CHOP _LP expstr _RP {
			if (doeval)
				$$ = chop($3,1);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _COMMAND _LP expstr _RP {
			if (doeval) {
				if (exprres == ERROR) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in command(), %s, command() call aborted, error in command: %s", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				} else {
					sysres = sys($3,COMMAND);
					if (!strcmp(sysres, err)) {
						/* pikt log error entry done in sys() */
						free(sysres);
						STRERR;
						$$ = mystrdup(err, _exprparse_);
					} else {
						$$ = sysres;
					}
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _ERRFNSTR _LP _RP {
			if (doeval)
				$$ = mystrdup(err, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _NILFNSTR _LP _RP {
			if (doeval)
				$$ = mystrdup(nil, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _FILEMODE _LP expstr _RP {
			if (doeval) {
				$$ = filemode($3);
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _MONTHNAME _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 1) && ($3 <= 12))
					$$ = mystrdup(monthname((int)$3), _exprparse_);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in monthname(), %s, non-integer or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _MONTHNAME _LP _RP {
			if (doeval)
				$$ = mystrdup(monthname(month(now())), _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _DAYNAME _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 1) && ($3 <= 7))
					$$ = mystrdup(dayname((int)$3), _exprparse_);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in dayname(), %s, non-integer or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _DAYNAME _LP _RP {
			if (doeval)
				$$ = mystrdup(dayname(weekday(now())), _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _DQUOTE _LP _RP {
			if (doeval)
				$$ = mystrdup(dquote, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _SQUOTE _LP _RP {
			if (doeval)
				$$ = mystrdup(squote, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _FIXED _LP expnum _CMA expnum _RP {
			if (doeval) {
				if (isint($5) && ($5 >= 0))
					$$ = fixed($3, (int)$5);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in fixed(), %s, non-integer or negative decimal operand: %f", ALMNAM, $5);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
			free($6);
		}
		| _FIXED _LP expnum _RP {
			if (doeval) {
				$$ = fixed($3,0);
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _FIXED _LP expstr _RP {
			if (doeval) {
				$$ = mystrdup($3, _exprparse_);
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _GRNAME _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXGID))
					$$ = grname((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in grname(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _GRNAME _LP expstr _RP {
			if (doeval)
				$$ = grname(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWNAME _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwname((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwname(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _PWNAME _LP expstr _RP {
			if (doeval)
				$$ = pwname(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _GRPASSWORD _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXGID))
					$$ = grpasswd((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in grpasswd(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _GRPASSWORD _LP expstr _RP {
			if (doeval)
				$$ = grpasswd(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWPASSWORD _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwpasswd((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwpasswd(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _PWPASSWORD _LP expstr _RP {
			if (doeval)
				$$ = pwpasswd(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWGECOS _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwgecos((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwgecos(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _PWGECOS _LP expstr _RP {
			if (doeval)
				$$ = pwgecos(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWDIR _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwdir((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwdir(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _PWDIR _LP expstr _RP {
			if (doeval)
				$$ = pwdir(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PWSHELL _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXUID))
					$$ = pwshell((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in pwshell(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _PWSHELL _LP expstr _RP {
			if (doeval)
				$$ = pwshell(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _GRMEMBERS _LP expnum _RP {
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXGID))
					$$ = grmem((int)$3, nil);
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in grmem(), %s, non-integral or out-of-bounds operand: %f", ALMNAM, $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($4);
		}
		| _GRMEMBERS _LP expstr _RP {
			if (doeval)
				$$ = grmem(NIL, $3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _HOSTNAME _LP _RP {
			if (doeval)
				$$ = mystrdup(hostname, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _INPUTLINE _LP _RP {
			if (doeval)
				$$ = mystrdup(inlinnow, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _LEFT _LP expstr _CMA expnum _RP {
			if (doeval) {
				if (isint($5) && ($5 >= 0)) {
					$$ = substring($3, 1, (size_t)$5);
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in left(), %s, non-integral or negative operand: %f", ALMNAM, $5);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($6);
		}
		| _LOWER _LP expstr _RP {
			if (doeval)
				$$ = lower($3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _UPPER _LP expstr _RP {
			if (doeval)
				$$ = upper($3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _PROPER _LP expstr _RP {
			if (doeval)
				$$ = proper($3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _REPEAT _LP expstr _CMA expnum _RP {
			if (doeval) {
				if (!isint($5) || ($5 < 0)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in repeat(), %s, non-integral or negative operand: %f", ALMNAM, $5);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				} else
					$$ = repeat($3, (int)$5);
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($6);
		}
		| _REPLACE _LP expstr _CMA expstr _CMA expnum _CMA expnum _RP {
			if (doeval) {
				if (!isint($7) || !isint($9) || ($7 <= 0) || ($9 < 0)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in replace(), %s, improper or non-integral operand(s): %f %f", ALMNAM, $7, $9);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				} else
					$$ = replace($3,$5,(int)$7,(int)$9);
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
			free($8);
			free($10);
		}
		| _SUBSTITUTE _LP expstr _CMA expstr _CMA expstr _CMA expnum _RP {
			if (doeval) {
				if (!isint($9) || ($9 <= 0)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in substitute(), %s, non-integral or negative operand: %f", ALMNAM, $9);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				} else {
					$$ = substitute($3,$5,$7,(int)$9);
					if (!strcmp($$,err)) {
						logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in substitute(), %s, return value exceeds %d char limit", ALMNAM, MAXBUF);
						STRERR;
						$$ = mystrdup(err, _exprparse_);
					}
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
			free($7);
			free($8);
			free($10);
		}
		| _SUBSTITUTE _LP expstr _CMA expstr _CMA expstr _RP {
			if (doeval) {
				$$ = substitute($3,$5,$7,0);
				if (!strcmp($$,err)) {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in substitute(), %s, return value exceeds %d char limit", ALMNAM, MAXBUF);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($5);
			free($6);
			free($7);
			free($8);
		}
		| _RIGHT _LP expstr _CMA expnum _RP {
			if (doeval) {
				if (isint($5) && ($5 >= 0)) {
					pos = strlen($3) - (int)$5 + 1;
					if (pos > 0) {
						$$ = substring($3, pos, strlen($3));
					} else {
						$$ = mystrdup($3, _exprparse_);
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in right(), %s, non-integral or negative operand: %f", ALMNAM, $5);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($6);
		}
		| _NEWLINE _LP _RP {
			if (doeval)
				$$ = my_char(10);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _SPACE _LP _RP {
			if (doeval)
				$$ = my_char(32);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _SUBSTRING _LP expstr _CMA expnum _CMA expnum _RP {
			if (doeval) {
				if (isint($5) && ($5 >= 0) && isint($7) && ($7 >= 0)) {
					$$ = substring($3, (int)$5, (size_t)$7);
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in substring(), %s, non-integral or negative decimal operands: %f %f", ALMNAM, $5, $7);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($6);
			free($8);
		}
		| _SUBSTRING _LP expstr _CMA expnum _RP {
			if (doeval) {
				if (isint($5) && ($5 >= 0))
					$$ = substring($3, (int)$5, strlen($3));
				else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in substring(), %s, non-integral or negative decimal operand: %f", ALMNAM, $5);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
			free($6);
		}
		| _TAB _LP _RP {
			if (doeval)
				$$ = my_char(9);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
		}
		| _TRIM _LP expstr _RP {
			if (doeval)
				$$ = trim($3,BOTH);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _LTRIM _LP expstr _RP {
			if (doeval)
				$$ = trim($3,LEFT);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _RTRIM _LP expstr _RP {
			if (doeval)
				$$ = trim($3,RIGHT);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _IFFNSTR _LP conditions {
			CHKIFNUMUB;
			ifnum++;
			doevals[ifnum] = doeval;
			if ((ifconds[ifnum] = $3))
				doeval = doevals[ifnum];
			else
				doeval = FALSE;
		} _CMA expstr iffnstrend {
			doeval = doevals[ifnum];
			CHKIFNUMLB;
			ifnum--;
			if (doeval)
				$$ = $3 ? mystrdup($6, _exprparse_) : mystrdup($7, _exprparse_);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($5);
			free($6);
			free($7);
		}
		| _BASENAME _LP expstr _RP {
			if (doeval)
				$$ = my_basename($3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _DIRNAME _LP expstr _RP {
			if (doeval)
				$$ = my_dirname($3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		| _REVERSE _LP expstr _RP {
			if (doeval)
				$$ = reverse($3);
			else
				$$ = mystrdup(nil, _exprparse_);
			free($1);
			free($2);
			free($3);
			free($4);
		}
		;

iffnnumend:	_CMA {
			if (ifconds[ifnum])
				doeval = FALSE;
			else
				doeval = doevals[ifnum];
		} expnum _RP {
			$$ = $3;
			free($1);
			free($4);
		}
		| _RP {
			$$ = NIL;
			free($1);
		}
		;

iffnstrend:	_CMA {
			if (ifconds[ifnum])
				doeval = FALSE;
			else
				doeval = doevals[ifnum];
		} expstr _RP {
			$$ = $3;
			free($1);
			free($4);
		}
		| _RP {
			$$ = mystrdup(nil, _exprparse_);
			free($1);
		}
		;

tok_as_str:	_LP { $$ = $1; /* the default action for all below */ }
		| _RP
		| _LB
		| _RB
		| _QLB
		| _CMA
		| _NOT
		| _AND
		| _OR
		| _ADD
		| _SUBTRACT
		| _MULTIPLY
		| _DIVIDE
		| _MODULUS
		| _EXPONENTIATE
		| _BITAND
		| _BITOR
		| _BITEXOR
		| _BITCOMP
		| _BITLSHFT
		| _BITRSHFT
		| _CONCATENATE
		| _EQ
		| _GT
		| _LT
		| _GE
		| _LE
		| _NGT
		| _NLT
		| _NE
		| _NE2
		| _RGXP
		| _RGXPI
		| _NRGXP
		| _NRGXPI
		| _SEQ
		| _SNE
		| _SLT
		| _SGT
		| _SLE
		| _SGE
		| _EXIST
		| _ZEROLEN
		| _REGFILE
		| _DIRECTORY
		| _CHARSPEC
		| _BLKSPEC
		| _PIPE
		| _SYMLINK
		| _SOCKET
		| _ABSVAL
		| _AGE
		| _ALARM
		| _AMPM
		| _ANDFN
		| _AVERAGE
		| _BASENAME
		| _CEILING
		| _CHAR
		| _CHECKSUM
		| _CHOP
		| _CODE
		| _COMMAND
		| _DATEVALUE
		| _DAY
		| _DAYNAME
		| _DAYNUMBER
		| _DAYSBETWEEN
		| _DEFINED
		| _DIRNAME
		| _DQUOTE
		| _ENDOFINPUT
		| _ERRFNNUM
		| _ERRFNSTR
		| _ERROR
		| _EVEN
		| _FALSEFN
		| _FCLOSE
		| _FILEMODE
		| _FILEINO
		| _FILELINKS
		| _FILEUID
		| _FILEGID
		| _FILESIZE
		| _FILEATIME
		| _FILEMTIME
		| _FILECTIME
		| _FIXED
		| _FOPEN
		| _GRNAME
		| _GRPASSWORD
		| _GRGID
		| _GRMEMBERS
		| _HOSTNAME
		| _HOUR
		| _IFFNNUM
		| _IFFNSTR
		| _INPUTLINE
		| _INPUTLENGTH
		| _INPUTNUMBER
		| _INTEGER
		| _ISITALNUM
		| _ISITALPHA
		| _ISITBLANK
		| _ISITCNTRL
		| _ISITDIGIT
		| _ISITGRAPH
		| _ISITLOWER
		| _ISITPRINT
		| _ISITPUNCT
		| _ISITSPACE
		| _ISITUPPER
		| _LEFT
		| _LENGTH
		| _LOWER
		| _LTRIM
		| _MAXIMUM
		| _MEDIAN
		| _MINIMUM
		| _MINUTE
		| _MODE
		| _MONTH
		| _MONTHNAME
		| _MONTHNUMBER
		| _NEWLINE
		| _NILFNNUM
		| _NILFNSTR
		| _NOTFN
		| _NOW
		| _ODD
		| _ORFN
		| _PARSE
		| _PCLOSE
		| _POPEN
		| _PROCID
		| _PPROCID
		| _PROPER
		| _PWNAME
		| _PWPASSWORD
		| _PWUID
		| _PWGID
		| _PWGECOS
		| _PWDIR
		| _PWSHELL
		| _RANDOM
		| _READ
		| _REPEAT
		| _REPLACE
		| _REVERSE
		| _RIGHT
		| _ROUND
		| _RSEARCH
		| _RTRIM
		| _SEARCH
		| _SECOND
		| _SPACE
		| _SQUOTE
		| _SUBSTITUTE
		| _SUBSTRING
		| _SYSTEM
		| _TAB
		| _TIMEVALUE
		| _TODAY
		| _TRIM
		| _TRUEFN
		| _TRUNC
		| _UPPER
		| _VALUE
		| _WEEKDAY
		| _WRITE
		| _YEAR
		| _YEARDAY
		| _YEARWEEK
		;

str_as_num:	_STR {
			if (doeval) {
				errno = 0;
				$$ = atof($1);
				if (errno) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, can't resolve string %s to number", ALMNAM, $1);
					exprstr = mystrdup(errstr, _exprparse_);
					NUMERR;
					$$ = ERROR;
				} else
					exprstr = $1;
			} else {
				exprstr = mystrdup(nil, _exprparse_);
				$$ = NIL;
				free($1);
			}
		}
		| _ERR {
			if (doeval) {
				exprstr = mystrdup(errstr, _exprparse_);
				$$ = ERROR;
			} else {
				exprstr = mystrdup(nil, _exprparse_);
				$$ = NIL;
			}
			free($1);
		}
		| _NIL {
			if (doeval) {
				exprstr = mystrdup(nilstr, _exprparse_);
				$$ = NIL;
			} else {
				exprstr = mystrdup(nil, _exprparse_);
				$$ = NIL;
			}
			free($1);
		}
		| _NULSTR {
			if (doeval) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, can't resolve null string to number", ALMNAM);
				exprstr = mystrdup("", _exprparse_);
				NUMERR;
				$$ = ERROR;
			} else {
				exprstr = mystrdup(nilstr, _exprparse_);
				$$ = NIL;
			}
		}
		| _ARYSTR _LB expnum _RB {
			--vapndx;
			if (doeval) {
				if (isint($3) && ($3 >= 0) && ($3 <= MAXARYNDX)) {
					if (vap[vapndx]) {
						if (vap[vapndx]->subary[SA((unsigned short)$3)] && vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)] && ($3 > 0)) {
							errno = 0;
							$$ = atof(vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)]);
							if (errno) {
								logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, can't resolve string %s to number", ALMNAM, vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)]);
								exprstr = mystrdup(errstr, _exprparse_);
								NUMERR;
								$$ = ERROR;
							} else {
								exprstr = (char *) mymalloc(MAXDBLSTR+1, _exprparse_);
								sprintf(exprstr, "%f", $$);
							}
							vap[vapndx] = (chrary *) NULL;
						} else {
							logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, attempt to reference nonexistent value for var $%s[%d]", ALMNAM, VARNAM($1), (int)$3);
							exprstr = mystrdup(errstr, _exprparse_);
							NUMERR;
							$$ = ERROR;
						}
					} else {
						errno = 0;
						$$ = atof(valscn[(int)$3]);
						if (errno) {
							logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, can't resolve string %s to number", ALMNAM, valscn[(unsigned short)$3]);
							exprstr = mystrdup(errstr, _exprparse_);
							NUMERR;
							$$ = ERROR;
						} else {
							exprstr = (char *) mymalloc(MAXDBLSTR+1, _exprparse_);
							sprintf(exprstr, "%f", $$);
						}
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, for var $%s[], non-integral array index: %f", ALMNAM, VARNAM($1), $3);
					exprstr = mystrdup(errstr, _exprparse_);
					NUMERR;
					$$ = ERROR;
				}
			} else {
				exprstr = mystrdup(nil, _exprparse_);
				$$ = NIL;
			}
			free($2);
			free($4);
		}
		;

num_as_str: 	_NUM {
			if (doeval) {
				sprintf(fmt, "%%.%df", dpls);
				sprintf(tmp, fmt, $1);
				if (isverboselog)
					logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, resolving number %f as string %s", ALMNAM, $1, tmp);
				$$ = mystrdup(tmp, _exprparse_);
			} else
				$$ = mystrdup(nil, _exprparse_);
		}
		| _INT {
			if (doeval) {
				sprintf(tmp, "%d", $1);
				if (isverboselog)
					logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, resolving number %ld as string %s", ALMNAM, $1, tmp);
				$$ = mystrdup(tmp, _exprparse_);
			} else
				$$ = mystrdup(nil, _exprparse_);
		}
		| _ARYNUM _LB expnum _RB {
			--vapndx;
			if (doeval) {
				if (isint($3) && ($3 > 0) && ($3 <= MAXARYNDX)) {
					if (vap[vapndx] && vap[vapndx]->subary[SA((unsigned short)$3)] && vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)]) {
						$$ = mystrdup(vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)], _exprparse_);
						if (isverboselog)
							logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, resolving number %s as string %s", ALMNAM, vap[vapndx]->subary[SA((unsigned short)$3)]->val[SN((unsigned short)$3)], $$);
						vap[vapndx] = (chrary *) NULL;
					} else {
						logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s, attempt to reference nonexistent value for var #%s[%d]", ALMNAM, VARNAM($1), (int)$3);
						STRERR;
						$$ = mystrdup(err, _exprparse_);
					}
				} else {
					logreturn(PIKTLOG, iserrdef ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s, for var #%s[], non-integral array index: %f", ALMNAM, VARNAM($1), $3);
					STRERR;
					$$ = mystrdup(err, _exprparse_);
				}
			} else
				$$ = mystrdup(nil, _exprparse_);
			free($2);
			free($4);
		}
		;

%%
