/*
   regexp.c -- PIKT regular expression routines

   $Header: /home/cvs/ucgsb/pikt/src/piktd/regexp.c,v 1.10 1999/10/04 05:22:51 harlan Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
*/

#include "../lib/common.h"
#include "piktd.h"

regmatch_t regmatch[MAXARYNDX+1];

/* when invoke #parse(), we inspect its second parameter (if any) to
   determine if this calls for a pattern match; if not, simply return with
   the value NIL indicating that a regexp pattern match does not apply;
   otherwise, this is a legitimate call for a regular expression pattern
   match, so pass the given string and regexp along to the GNU rx routines;
   if we have parenthesized patterns, and we have a match, copy the matched
   substrings to the valscan[] array as necessary; set matches equal to
   the number of such matches; return TRUE or FALSE, depending on whether
   or not the given string pattern matches the regexp
 */
int
regexp (char *str, char *exp, int cflags, int isparsefn)
{
	regex_t preg;
	int regerrcode;
	char regerrmsg[MAXLINE+1];
	register int i;
	register char c;
	int islp;
	int openlp;
	int nmatch;
	int len;

	len = 0;
	freevalscn();
	islp = FALSE;
	openlp = FALSE;
	nmatch = 1;
	i = 0;
	while ((c = *(exp+i))) {
		i++;
		switch (c) {
			case '\\':
				islp = FALSE;
				if (*(exp+i))
					i++;
				break;
			case '(':
				islp = TRUE;
				openlp = TRUE;
				break;
			case ')':
				if (openlp && !islp) {
					nmatch++;
					openlp = FALSE;
				}
				/* no break here! */
			default:
				islp = FALSE;
				break;
		}
	}
	if (isparsefn && (nmatch == 1))
		return NIL;
	if ((regerrcode = regcomp(&preg, exp, cflags))) {
		regerror(regerrcode, &preg, regerrmsg, MAXLINE);
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in regexp(), %s, regcomp() error for regexp %s: %s", ALMNAM, exp, regerrmsg);
		if (doeval) exprres = ERROR;
		return FALSE;
	}
	regerrcode = regexec(&preg, str, (size_t) nmatch, regmatch, 0 /* REG_NOTBOL|REG_NOTEOL */);
	regfree(&preg);
	if (regerrcode) {
		matches = 0;
		return FALSE;
	} else {
		matches = nmatch-1;
		for (i=0; i<=matches; i++) {
			len = regmatch[i].rm_eo - regmatch[i].rm_so;
			valscn[i] = (char *) mymalloc((size_t)(len+1), "regcomp");
			strncpy(valscn[i], str+regmatch[i].rm_so, (size_t)len);
			*(valscn[i]+len) = '\0';
			if ((i == 0) && (POSTYP == REGEXPR)) {
				if (*(valscn[0]+len-1) == '\n')
					*(valscn[0]+len-1) = '\0';
			}
		}
		/* when parsing the input line using regexp, chop the
		   linefeed, if present, from the last field (this was
		   done for valscn[0] above)
		 */
		if (POSTYP == REGEXPR) {
			if (*(valscn[i-1]+len-1) == '\n')
				*(valscn[i-1]+len-1) = '\0';
		}
		return TRUE;
	}
}
