/*
   signal.c -- PIKT pikt & piktd signal handling routines

   $Header: /home/cvs/ucgsb/pikt/src/piktd/signal.c,v 1.13 1999/10/09 03:44:54 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
*/

#include "../lib/common.h"

#include "piktd.h"

struct sigaction sigchld_act;
struct sigaction sigterm_act;

void sig_chld(int);
void sig_term(int);

/* initialize the signal handlers
 */
void
sig_init (void)
{

#ifdef SA_SIGINFO
	/* HMS: Why bother with sa_sigaction if the handler takes no args? */
	sigchld_act.sa_sigaction = (void *) sig_chld;
	sigchld_act.sa_flags = SA_SIGINFO
#ifdef SA_NOCLDWAIT
			       | SA_NOCLDWAIT
#endif
			       ;
#else
	sigchld_act.sa_handler = sig_chld;
#endif
	sigemptyset(&sigchld_act.sa_mask);
	if (sigaction(SIGCHLD, &sigchld_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGCHLD");
	}

#ifdef SA_SIGINFO
	sigterm_act.sa_sigaction = (void *) sig_term;
	sigterm_act.sa_flags = SA_SIGINFO;
#else
	sigterm_act.sa_handler = sig_term;
#endif
	sigemptyset(&sigterm_act.sa_mask);
	if (sigaction(SIGTERM, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGTERM");
	}
	if (sigaction(SIGINT, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGINT");
	}
	if (sigaction(SIGQUIT, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGQUIT");
	}

}

/* SIGCHLD handler
 */
void
sig_chld (int sig)
{

	/* log reason for any abnormal child terminations? */
	/* wait((int *)NULL); -- this caused no end of difficulties!! */
	return;
}

/* SIGTERM handler--on program termination:  remove the alert lock file,
   any exec lock file, and any history temp file, for pikt; or remove the
   daemon lock file, and the alerts file, for piktd
 */
void
sig_term (int sig)
{
	struct stat fbuf;

	#ifdef PIKTD
	if (rmlocks(FALSE, TRUE) == ERROR)
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in sig_term(), rmlocks() error");
	if (piktdalt && (stat(piktdalt, &fbuf) == 0))
		remove(piktdalt);
	#else
	if (rmlocks(interactive, TRUE) == ERROR)
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in sig_term(), rmlocks() error");
	if (hsttmpfile && (stat(hsttmpfile, &fbuf) == 0))
		remove(hsttmpfile);
	#endif
	exit(0);
}
